@@ -1,4 +1,3 @@
-import io
 import json
 from enum import Enum, auto
 
@@ -7,19 +6,12 @@ import pytest
 
 import streamson
 
-DATA = [b'{"users": ["john", "carl", "bob"]}']
-
 
 class Kind(Enum):
     FD = auto()
     ITER = auto()
 
 
-@pytest.fixture
-def io_reader() -> io.BytesIO:
-    return io.BytesIO(b'{"users": ["john", "carl", "bob"]}')
-
-
 @pytest.mark.parametrize(
     "kind,convert,extract_path",
     [
@@ -51,10 +43,10 @@ def io_reader() -> io.BytesIO:
         "iter-hyperjson-nopath",
     ],
 )
-def test_simple(io_reader, kind, convert, extract_path):
+def test_simple(io_reader, data, kind, convert, extract_path):
     matcher = streamson.SimpleMatcher('{"users"}[]')
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
     assert next(extracted) == ('{"users"}[0]' if extract_path else None, convert('"john"'))
@@ -96,35 +88,32 @@ def test_simple(io_reader, kind, convert, extract_path):
         "iter-hyperjson-nopath",
     ],
 )
-def test_depth(io_reader, kind, convert, extract_path):
-    matcher = streamson.DepthMatcher(1)
+def test_depth(io_reader, data, kind, convert, extract_path):
+    matcher = streamson.DepthMatcher("1")
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
-    assert next(extracted) == ('{"users"}[0]' if extract_path else None, convert('"john"'))
-    assert next(extracted) == ('{"users"}[1]' if extract_path else None, convert('"carl"'))
-    assert next(extracted) == ('{"users"}[2]' if extract_path else None, convert('"bob"'))
     assert next(extracted) == (
         '{"users"}' if extract_path else None,
         convert('["john", "carl", "bob"]'),
     )
+    assert next(extracted) == (
+        '{"groups"}' if extract_path else None,
+        convert('["admins", "users"]'),
+    )
     with pytest.raises(StopIteration):
         next(extracted)
 
-    matcher = streamson.DepthMatcher(0, 1)
+    matcher = streamson.DepthMatcher("0")
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         io_reader.seek(0)
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
-    assert next(extracted) == (
-        '{"users"}' if extract_path else None,
-        convert('["john", "carl", "bob"]'),
-    )
     assert next(extracted) == (
         "" if extract_path else None,
-        convert('{"users": ["john", "carl", "bob"]}'),
+        convert('{"users": ["john", "carl", "bob"], "groups": ["admins", "users"]}'),
     )
 
     with pytest.raises(StopIteration):
@@ -162,19 +151,15 @@ def test_depth(io_reader, kind, convert, extract_path):
         "iter-hyperjson-nopath",
     ],
 )
-def test_invert(io_reader, kind, convert, extract_path):
-    matcher = ~streamson.DepthMatcher(2)
+def test_invert(io_reader, data, kind, convert, extract_path):
+    matcher = ~streamson.DepthMatcher("2")
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
-    assert next(extracted) == (
-        '{"users"}' if extract_path else None,
-        convert('["john", "carl", "bob"]'),
-    )
     assert next(extracted) == (
         "" if extract_path else None,
-        convert('{"users": ["john", "carl", "bob"]}'),
+        convert('{"users": ["john", "carl", "bob"], "groups": ["admins", "users"]}'),
     )
 
 
@@ -209,11 +194,11 @@ def test_invert(io_reader, kind, convert, extract_path):
         "iter-hyperjson-nopath",
     ],
 )
-def test_all(io_reader, kind, convert, extract_path):
+def test_all(io_reader, data, kind, convert, extract_path):
     matcher = streamson.SimpleMatcher('{"users"}[]') & streamson.SimpleMatcher("{}[1]")
 
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
     assert next(extracted) == ('{"users"}[1]' if extract_path else None, convert('"carl"'))
@@ -253,20 +238,25 @@ def test_all(io_reader, kind, convert, extract_path):
         "iter-hyperjson-nopath",
     ],
 )
-def test_any(io_reader, kind, convert, extract_path):
-    matcher = streamson.DepthMatcher(2, 2) | streamson.SimpleMatcher('{"users"}')
+def test_any(io_reader, data, kind, convert, extract_path):
+    matcher = streamson.DepthMatcher("2-2") | streamson.SimpleMatcher('{"users"}')
 
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
-    assert next(extracted) == ('{"users"}[0]' if extract_path else None, convert('"john"'))
-    assert next(extracted) == ('{"users"}[1]' if extract_path else None, convert('"carl"'))
-    assert next(extracted) == ('{"users"}[2]' if extract_path else None, convert('"bob"'))
     assert next(extracted) == (
         '{"users"}' if extract_path else None,
         convert('["john", "carl", "bob"]'),
     )
+    assert next(extracted) == (
+        '{"groups"}[0]' if extract_path else None,
+        convert('"admins"'),
+    )
+    assert next(extracted) == (
+        '{"groups"}[1]' if extract_path else None,
+        convert('"users"'),
+    )
 
     with pytest.raises(StopIteration):
         next(extracted)
@@ -303,21 +293,23 @@ def test_any(io_reader, kind, convert, extract_path):
         "iter-hyperjson-nopath",
     ],
 )
-def test_complex(io_reader, kind, convert, extract_path):
-    matcher = (streamson.DepthMatcher(2, 2) | streamson.SimpleMatcher('{"users"}')) & ~streamson.SimpleMatcher(
-        '{"users"}[0]'
+def test_complex(io_reader, data, kind, convert, extract_path):
+    matcher = (streamson.DepthMatcher("2-2") | streamson.SimpleMatcher('{"users"}')) & ~streamson.SimpleMatcher(
+        '{"groups"}[0]'
     )
 
     if kind == Kind.ITER:
-        extracted = streamson.extract_iter((e for e in DATA), matcher, convert, extract_path)
+        extracted = streamson.extract_iter((e for e in data), matcher, convert, extract_path)
     elif kind == Kind.FD:
         extracted = streamson.extract_fd(io_reader, matcher, 5, convert, extract_path)
-    assert next(extracted) == ('{"users"}[1]' if extract_path else None, convert('"carl"'))
-    assert next(extracted) == ('{"users"}[2]' if extract_path else None, convert('"bob"'))
     assert next(extracted) == (
         '{"users"}' if extract_path else None,
         convert('["john", "carl", "bob"]'),
     )
+    assert next(extracted) == (
+        '{"groups"}[1]' if extract_path else None,
+        convert('"users"'),
+    )
 
     with pytest.raises(StopIteration):
         next(extracted)
